@@ -20,6 +20,7 @@
 
 #define _XOPEN_SOURCE	/* strptime proto, but this hides other prototypes */
 #define _GNU_SOURCE	/* get all the other prototypes */
+#define __EXTENSIONS__	/* for solaris to get back strdup and strcasecmp */
 
 #include "config.h"
 #include "actions.h"
@@ -36,6 +37,7 @@
 
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-setting.h>
+#include <gphoto2/gphoto2-filesys.h>
 #include <gphoto2/globals.h>
 
 #ifdef HAVE_AA
@@ -50,6 +52,12 @@
 #define CRU(result,file) {int r=(result); if (r<0) {gp_file_unref(file);return r;}}
 #define CL(result,list)  {int r=(result); if (r<0) {gp_list_free(list); return r;}}
 
+#ifdef __GNUC__
+#define __unused__ __attribute__((unused))
+#else
+#define __unused__
+#endif
+
 int
 delete_all_action (GPParams *p)
 {
@@ -64,11 +72,11 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 
 	gp_log (GP_LOG_DEBUG, "main", "Uploading file...");
 
-	CR (gp_file_new (&file));
+	CR (gp_file_new_from_fd (&file, -1));
 	res = gp_file_open (file, path);
-	if (res < 0) {
+	if (res < GP_OK) {
 		gp_file_unref (file);
-		return (res);
+		return res;
 	}
 
 	/* Check if the user specified a filename */
@@ -79,7 +87,6 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 			return (res);
 		}
 	}
-
 	res = gp_camera_folder_put_file (p->camera, folder, file,
 					 p->context);
 	gp_file_unref (file);
@@ -169,7 +176,7 @@ list_folders_action (GPParams *p)
 	CameraList *list;
 	int count;
 	const char *name;
-	unsigned int i;
+	int i;
 	
 	CR (gp_list_new (&list));
 
@@ -195,7 +202,7 @@ list_files_action (GPParams *p)
 	CameraList *list;
 	int count, filecount;
 	const char *name;
-	unsigned int i;
+	int i;
 
 	CR (gp_list_new (&list));
 	CL (gp_camera_folder_list_files (p->camera, p->folder, list,
@@ -475,7 +482,7 @@ print_exif_action (GPParams *p, const char *filename)
         CRU (gp_camera_file_get (p->camera, p->folder, filename,
 				 GP_FILE_TYPE_EXIF, file, p->context), file);
         CRU (gp_file_get_data_and_size (file, &data, &size), file);
-        ed = exif_data_new_from_data (data, size);
+        ed = exif_data_new_from_data ((unsigned char *)data, size);
         gp_file_unref (file);
         if (!ed) {
                 gp_context_error (p->context, _("Could not parse EXIF data."));
@@ -529,7 +536,7 @@ list_cameras_action (GPParams *p)
 	int r = GP_OK, n, i;
 	CameraAbilities a;
 
-	r = gp_abilities_list_count (p->abilities_list);
+	r = gp_abilities_list_count (gp_params_abilities_list(p));
 	if (r < 0)
 		return (r);
 	if (p->flags & FLAGS_QUIET)
@@ -540,7 +547,7 @@ list_cameras_action (GPParams *p)
 	}
 	n = r;
 	for (i = 0; i < n; i++) {
-		r = gp_abilities_list_get_abilities (p->abilities_list,
+		r = gp_abilities_list_get_abilities (gp_params_abilities_list(p),
 						     i, &a);
 		if (r < 0)
 			break;
@@ -565,7 +572,7 @@ list_cameras_action (GPParams *p)
 	return (r);
 }
 
-static void
+void
 _get_portinfo_list (GPParams *p) {
 	int count, result;
 	GPPortInfoList *list = NULL;
@@ -622,17 +629,13 @@ auto_detect_action (GPParams *p)
 {
 	int x, count;
         CameraList *list;
-        CameraAbilitiesList *al = NULL;
         const char *name = NULL, *value = NULL;
 
 	_get_portinfo_list (p);
 	count = gp_port_info_list_count (p->portinfo_list);
 
 	CR (gp_list_new (&list));
-        gp_abilities_list_new (&al);
-        gp_abilities_list_load (al, p->context);
-        gp_abilities_list_detect (al, p->portinfo_list, list, p->context);
-        gp_abilities_list_free (al);
+        gp_abilities_list_detect (gp_params_abilities_list(p), p->portinfo_list, list, p->context);
 
         CL (count = gp_list_count (list), list);
 
@@ -839,8 +842,8 @@ action_camera_set_model (GPParams *p, const char *model)
 	CameraAbilities a;
 	int m;
 
-	CR (m = gp_abilities_list_lookup_model (p->abilities_list, model));
-	CR (gp_abilities_list_get_abilities (p->abilities_list, m, &a));
+	CR (m = gp_abilities_list_lookup_model (gp_params_abilities_list(p), model));
+	CR (gp_abilities_list_get_abilities (gp_params_abilities_list(p), m, &a));
 	CR (gp_camera_set_abilities (p->camera, a));
 	gp_setting_set ("gphoto2", "model", a.model);
 
@@ -868,7 +871,7 @@ set_filename_action (GPParams *p, const char *filename)
 #define CHECK_NULL(x) { if (x == NULL) { return(-1); /* FIXME: what code? */ } }
 
 int
-print_version_action (GPParams *p)
+print_version_action (GPParams __unused__ *p)
 {
 	int n;
 	const char *port_message =
@@ -879,7 +882,7 @@ print_version_action (GPParams *p)
 #endif
 	printf (_("gphoto2 %s\n"
 		  "\n"
-		  "Copyright (c) 2000-2004 Lutz Mueller and others\n"
+		  "Copyright (c) 2000-%d Lutz Mueller and others\n"
 		  "%s"
 		  "\n"
 		  "gphoto2 comes with NO WARRANTY, to the extent permitted by law. You may\n"
@@ -888,6 +891,7 @@ print_version_action (GPParams *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
+		2006, /* year of release! */
 		port_message
 		);
 
@@ -901,7 +905,7 @@ print_version_action (GPParams *p)
 	  v = func(GP_VERSION_SHORT);
 	  CHECK_NULL (v);
 	  CHECK_NULL (v[0]);
-	  printf ("%-17s %-12s ", name, v[0]);
+	  printf ("%-15s %-14s ", name, v[0]);
 	  for (i = 1; v[i] != NULL; i++) {
 		  if (v[i+1] != NULL)
 			  printf ("%s, ", v[i]);
@@ -931,7 +935,7 @@ action_camera_capture_preview (GPParams *p)
 		return (r);
 	}
 
-	r = save_camera_file_to_file (NULL, file);
+	r = save_camera_file_to_file (NULL, file, NULL);
 	gp_file_unref (file);
 	if (r < 0) 
 		return (r);
@@ -973,6 +977,95 @@ action_camera_wait_event (GPParams *p)
 	return GP_OK;
 }
 
+int
+print_storage_info (GPParams *p)
+{
+	int			ret, i, nrofsinfos;
+	CameraStorageInformation	*sinfos;
+
+	ret = gp_camera_get_storageinfo (p->camera, &sinfos, &nrofsinfos, p->context);
+	if (ret != GP_OK) {
+		if (ret == GP_ERROR_NOT_SUPPORTED)
+			printf (_("Getting storage information not supported for this camera.\n"));
+		return ret;
+	}
+	for (i=0;i<nrofsinfos;i++) {
+		printf ("[Storage %d]\n", i);
+		if (sinfos[i].fields & GP_STORAGEINFO_LABEL)
+			printf ("label=%s\n", sinfos[i].label);
+		if (sinfos[i].fields & GP_STORAGEINFO_DESCRIPTION)
+			printf ("description=%s\n", sinfos[i].description);
+		if (sinfos[i].fields & GP_STORAGEINFO_BASE)
+			printf ("basedir=%s\n", sinfos[i].basedir);
+		if (sinfos[i].fields & GP_STORAGEINFO_ACCESS) {
+			printf ("access=%d ", sinfos[i].access);
+			switch (sinfos[i].access) {
+			case GP_STORAGEINFO_AC_READWRITE:
+				printf (_("Read-Write"));
+				break;
+			case GP_STORAGEINFO_AC_READONLY:
+				printf (_("Read-Only"));
+				break;
+			case GP_STORAGEINFO_AC_READONLY_WITH_DELETE:
+				printf (_("Read-only with delete"));
+				break;
+			default:
+				printf (_("Unknown"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_STORAGETYPE) {
+			printf ("type=%d ", sinfos[i].type);
+			switch (sinfos[i].type) {
+			default:
+			case GP_STORAGEINFO_ST_UNKNOWN:
+				printf ( _("Unknown"));
+				break;
+			case GP_STORAGEINFO_ST_FIXED_ROM:
+				printf ( _("Fixed ROM"));
+				break;
+			case GP_STORAGEINFO_ST_REMOVABLE_ROM:
+				printf ( _("Removable ROM"));
+				break;
+			case GP_STORAGEINFO_ST_FIXED_RAM:
+				printf ( _("Fixed RAM"));
+				break;
+			case GP_STORAGEINFO_ST_REMOVABLE_RAM:
+				printf ( _("Removable RAM"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_FILESYSTEMTYPE) {
+			printf ("fstype=%d ", sinfos[i].type);
+			switch (sinfos[i].fstype) {
+			default:
+			case GP_STORAGEINFO_FST_UNDEFINED:
+				printf ( _("Undefined"));
+				break;
+			case GP_STORAGEINFO_FST_GENERICFLAT:
+				printf ( _("Generic Flat"));
+				break;
+			case GP_STORAGEINFO_FST_GENERICHIERARCHICAL:
+				printf ( _("Generic Hierarchical"));
+				break;
+			case GP_STORAGEINFO_FST_DCF:
+				printf ( _("Camera layout (DCIM)"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_MAXCAPACITY)
+			printf ("totalcapacity=%ld KB\n", sinfos[i].capacitykbytes);
+		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEKBYTES)
+			printf ("free=%ld KB\n", sinfos[i].freekbytes);
+		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEIMAGES)
+			printf ("freeimages=%ld\n", sinfos[i].freeimages);
+	}
+	return GP_OK;
+}
+
 int
 override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 			int usb_vendor_modified, int usb_product_modified)
@@ -983,9 +1076,13 @@ override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 
 	CR (gp_abilities_list_new (&al));
 
-	n = gp_abilities_list_count (p->abilities_list);
+	/* The override_usbides_action() function is a notable
+	 * exception to the rule that one is not supposed to use
+	 * p->_abilities_list directly, because it has to and does so
+	 * in a safe way. */
+	n = gp_abilities_list_count (gp_params_abilities_list(p));
 	for (i = 0; i < n; i++) {
-		r = gp_abilities_list_get_abilities (p->abilities_list, i,
+		r = gp_abilities_list_get_abilities (gp_params_abilities_list(p), i,
 						     &a);
 		if (r < 0)
 			continue;
@@ -1001,14 +1098,15 @@ override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 		}
 		gp_abilities_list_append (al, a);
 	}
-	gp_abilities_list_free (p->abilities_list);
-	p->abilities_list = al;
+
+	gp_abilities_list_free (p->_abilities_list);
+	p->_abilities_list = al;
 
 	return (GP_OK);
 }
 
 /* time zero for debug log time stamps */
-struct timeval glob_tv_zero = { 0, 0 };
+static struct timeval glob_tv_zero = { 0, 0 };
 
 static void
 #ifdef __GNUC__
@@ -1019,20 +1117,22 @@ debug_func (GPLogLevel level, const char *domain, const char *format,
 {
 	struct timeval tv;
 	long sec, usec;
+	FILE *logfile = (data != NULL)?(FILE *)data:stderr;
 
 	gettimeofday (&tv,NULL);
 	sec = tv.tv_sec  - glob_tv_zero.tv_sec;
 	usec = tv.tv_usec - glob_tv_zero.tv_usec;
 	if (usec < 0) {sec--; usec += 1000000L;}
-	fprintf (stderr, "%li.%06li %s(%i): ", sec, usec, domain, level);
-	vfprintf (stderr, format, args);
-	fputc ('\n', stderr);
+	fprintf (logfile, "%li.%06li %s(%i): ", sec, usec, domain, level);
+	vfprintf (logfile, format, args);
+	fputc ('\n', logfile);
 }
 
 int
-debug_action (GPParams *p)
+debug_action (GPParams *p, const char *debug_logfile_name)
 {
 	int n;
+	FILE *logfile = NULL;
 
 	/* make sure we're only executed once */
 	static int debug_flag = 0;
@@ -1040,9 +1140,19 @@ debug_action (GPParams *p)
 		return(GP_OK);
 	debug_flag = 1;
 
+	if (debug_logfile_name != NULL) {
+	  /* FIXME: Handle fopen() error besides using stderr? */
+	  logfile = fopen(debug_logfile_name, "a");
+	}
+	if (logfile == NULL) {
+	  logfile = stderr;
+	}
+	setbuf(logfile, NULL);
+	setbuf(stdout, NULL);
+
 	gettimeofday (&glob_tv_zero, NULL);
 
-	CR (p->debug_func_id = gp_log_add_func (GP_LOG_ALL, debug_func, NULL));
+	CR (p->debug_func_id = gp_log_add_func (GP_LOG_ALL, debug_func, (void *) logfile));
 	gp_log (GP_LOG_DEBUG, "main", _("ALWAYS INCLUDE THE FOLLOWING LINES "
 					"WHEN SENDING DEBUG MESSAGES TO THE "
 					"MAILING LIST:"));
@@ -1064,6 +1174,25 @@ debug_action (GPParams *p)
 	  }
 	}
 
+	if (1) {
+		/* This is internal debug stuff for developers - no
+		 * need for translation IMHO */
+		const char *iolibs = getenv("IOLIBS");
+		const char *camlibs = getenv("CAMLIBS");
+		if (camlibs) {
+			gp_log (GP_LOG_DEBUG, "main", "CAMLIBS = '%s'", camlibs);
+		} else {
+			gp_log (GP_LOG_DEBUG, "main", 
+				"CAMLIBS env var not set, using compile-time default instead");
+		}
+		if (iolibs) {
+			gp_log (GP_LOG_DEBUG, "main", "IOLIBS = '%s'", iolibs);
+		} else {
+			gp_log (GP_LOG_DEBUG, "main", 
+				"IOLIBS env var not set, using compile-time default instead");
+		}
+	}
+
 	return (GP_OK);
 }
 
@@ -1088,11 +1217,12 @@ display_widgets (CameraWidget *widget, char *prefix) {
 	n = gp_widget_count_children (widget);
 
 	newprefix = malloc(strlen(prefix)+1+strlen(uselabel)+1);
+	if (!newprefix)
+		abort();
 	sprintf(newprefix,"%s/%s",prefix,uselabel);
 
-	if ((type != GP_WIDGET_WINDOW) && (type != GP_WIDGET_SECTION)) {
+	if ((type != GP_WIDGET_WINDOW) && (type != GP_WIDGET_SECTION))
 		printf("%s\n",newprefix);
-	}
 	for (i=0; i<n; i++) {
 		CameraWidget *child;
 	
@@ -1101,6 +1231,7 @@ display_widgets (CameraWidget *widget, char *prefix) {
 			continue;
 		display_widgets (child, newprefix);
 	}
+	free(newprefix);
 }
 
 
@@ -1354,18 +1485,19 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		int	t;
 
 		t = 2;
-		if (	strcasecmp (value, "off")	|| strcasecmp (value, "no")	||
-			strcasecmp (value, "false")	|| strcmp (value, "0")		||
-			strcasecmp (value, _("off"))	|| strcasecmp (value, _("no"))	||
-			strcasecmp (value, _("false"))
+		if (	!strcasecmp (value, "off")	|| !strcasecmp (value, "no")	||
+			!strcasecmp (value, "false")	|| !strcmp (value, "0")		||
+			!strcasecmp (value, _("off"))	|| !strcasecmp (value, _("no"))	||
+			!strcasecmp (value, _("false"))
 		)
 			t = 0;
-		if (	strcasecmp (value, "on")	|| strcasecmp (value, "yes")	||
-			strcasecmp (value, "true")	|| strcmp (value, "0")		||
-			strcasecmp (value, _("on"))	|| strcasecmp (value, _("yes"))	||
-			strcasecmp (value, _("true"))
+		if (	!strcasecmp (value, "on")	|| !strcasecmp (value, "yes")	||
+			!strcasecmp (value, "true")	|| !strcmp (value, "1")		||
+			!strcasecmp (value, _("on"))	|| !strcasecmp (value, _("yes"))	||
+			!strcasecmp (value, _("true"))
 		)
 			t = 1;
+		fprintf (stderr," value %s, t %d\n", value, t);
 		if (t == 2) {
 			gp_context_error (p->context, _("The passed value %s is not a valid toggle value."), value);
 			ret = GP_ERROR_BAD_PARAMETERS;
@@ -1450,3 +1582,11 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 	gp_widget_free (rootconfig);
 	return (ret);
 }
+
+
+/*
+ * Local Variables:
+ * c-file-style:"linux"
+ * indent-tabs-mode:t
+ * End:
+ */
