@@ -105,7 +105,7 @@ static int shell_construct_path (const char *folder_orig, const char *rel_path,
 typedef int (* ShellFunction) (Camera *camera, const char *arg);
 
 typedef struct _ShellFunctionTable ShellFunctionTable;
-struct _ShellFunctionTable {
+static const struct _ShellFunctionTable {
 	const char *command;
 	ShellFunction function;
 	const char *description;
@@ -148,7 +148,7 @@ struct _ShellFunctionTable {
 
 /* Local globals */
 #define SHELL_PROMPT "gphoto2: {%s} %s> "
-int 	shell_done		= 0;
+static int 	shell_done		= 0;
 
 static unsigned int
 shell_arg_count (const char *args)
@@ -575,7 +575,8 @@ shell_lcd (Camera __unused__ *camera, const char *arg)
 			cli_error_print (_("Could not find home directory."));
 			return (GP_OK); 
 		}
-		strcpy (new_cwd, getenv ("HOME"));
+		strncpy (new_cwd, getenv ("HOME"), sizeof(new_cwd)-1);
+		new_cwd[sizeof(new_cwd)-1] = '\0';
 	} else
 		shell_construct_path (cwd, arg, new_cwd, NULL);
 
